@@ -117,7 +117,6 @@ def build(input_graph_string, timestamp):
     anm.add_overlay("ospf")
     anm.add_overlay("isis")
 
-
     ank.copy_attr_from(G_in, G_phy, "include_csr") #TODO: find more elegant passing method from input to compiler
 
     if igp == "ospf":
@@ -126,16 +125,14 @@ def build(input_graph_string, timestamp):
         build_isis(anm)
     build_bgp(anm)
 
+    #TODO: need to allocate loopback indexes to remaining loopbacks
+    #TODO: need to add loopback zero as loopback with index 0, and add anm selector for loopback0
+
     #TODO: provide an ANM wide function that allocates interfaces
     #TODO: work out why some interfaces in bgp graph in vis have node data....
-    for node in G_phy:
-        for interface in node:
-            interface.speed = 102
-
 
     return anm
 
-
 def boundary_nodes(G, nodes):
     #TODO: move to utils
     """ returns nodes at boundary of G
@@ -164,6 +161,10 @@ def build_vrf(anm):
     for node in G_vrf:
         interface = node.add_interface(name = "test")
 
+    for node in G_vrf.nodes(vrf_role = "CE"):
+        if not node.vrf:
+            node.vrf = "default_vrf"
+
 # set default
     non_ce_nodes = [n for n in G_vrf if n.vrf_role != "CE"]
     for n in non_ce_nodes:
@@ -175,6 +176,7 @@ def build_vrf(anm):
         else:
             n.vrf_role = "P" # default role
 
+
     """TODO: sanity checks:
     - vrf only set on CE nodes
     """
@@ -201,16 +203,23 @@ def build_vrf(anm):
     for node in G_vrf.nodes(vrf_role = "PE"):
         node_vrf_names = set(n.vrf for n in node.neighbors(vrf_role = "CE"))
         for vrf_name in node_vrf_names:
-            node.add_loopback(vrf_name = vrf_name)
+            node.add_loopback(vrf_name = vrf_name, 
+                    description = "loopback for vrf %s" % vrf_name)
+
+    for node in G_vrf.nodes(vrf_role = "CE"):
+            node.add_loopback(vrf_name = n.vrf_name, 
+                    description = "loopback for vrf %s" % n.vrf_name)
+
         #for peering in node.edges():
             #node.add_loopback(color = 1, vrf_peer = peering.dst)
         #for interface in node.interfaces(type = "loopback", color = 1):
             #print interface, interface.type, interface.color
 
     for node in G_vrf:
-        for i in node.interfaces("vrf_peer"):
-            #print i, i.vrf_peer, i.color
-            pass
+        for index, i in enumerate(node.interfaces("is_loopback", "vrf_name")):
+            # map (0, 1, 2, 3, ...) -> (101, 102, 103, 104, ...)
+            i.index = 100 + index + 1
+            i.route_target = "default_rt"
 
 # Create route-targets
 #TODO: this should be per ASN
@@ -402,7 +411,6 @@ def build_ip6(anm):
     G_ip6.add_edges_from(G_ip4.edges())
     ipv6.allocate_ips(G_ip6) 
 
-
 def build_ipv4(anm, infrastructure = True):
     G_ip4 = anm.add_overlay("ipv4")
     G_in = anm['input']
