@@ -261,9 +261,21 @@ class IosBaseCompiler(RouterCompiler):
     lo_interface = "%s%s"  % (lo_interface_prefix, 0)
 
     def compile(self, node):
+        phy_node = self.anm['phy'].node(node)
+
+        if node in self.anm['ospf']:
+            node.ospf.use_ipv4 = phy_node.use_ipv4
+            node.ospf.use_ipv6 = phy_node.use_ipv6
+
+        if node in self.anm['isis']:
+            node.isis.use_ipv4 = phy_node.use_ipv4
+            node.isis.use_ipv6 = phy_node.use_ipv6
+
         super(IosBaseCompiler, self).compile(node)
         if node in self.anm['isis']:
             self.isis(node)
+
+
         node.label = self.anm['phy'].node(node).label
         self.vrf(node)
         
@@ -276,7 +288,7 @@ class IosBaseCompiler(RouterCompiler):
             ipv4_cidr = address_prefixlen_to_network(ipv4_address, ipv4_loopback_subnet.prefixlen)
         if node.ip.use_ipv6:
             ipv6_node = self.anm['ipv6'].node(node)
-            ipv6_address = address_prefixlen_to_network(ipv6_node.loopback, 126)
+            ipv6_address = address_prefixlen_to_network(ipv6_node.loopback, 128)
 
 #TODO: strip out returns from super
         super(IosBaseCompiler, self).interfaces(node)
@@ -288,12 +300,16 @@ class IosBaseCompiler(RouterCompiler):
             ospf_link = G_ospf.edge(interface._edge_id) # find link in OSPF with this ID
             if ospf_link:
                 interface['ospf_cost'] = ospf_link.cost
+                interface['ospf_use_ivp4'] = node.ospf.use_ipv4
+                interface['ospf_use_ivp6'] = node.ospf.use_ipv6
             isis_link = G_isis.edge(interface._edge_id) # find link in OSPF with this ID
             if isis_link: # only configure if has ospf interface
                 interface['isis'] = True
                 isis_node = G_isis.node(node)
                 interface['isis_process_id'] = isis_node.process_id  #TODO: should this be from the interface?
                 interface['isis_metric'] = isis_link.metric  #TODO: should this be from the interface?
+                interface['isis_use_ivp4'] = node.isis.use_ipv4
+                interface['isis_use_ivp6'] = node.isis.use_ipv6
 
 #TODO: update this to new format
         is_isis_node = bool(G_isis.node(node)) # if node is in ISIS graph
@@ -314,6 +330,15 @@ class IosBaseCompiler(RouterCompiler):
 
     def ospf(self, node):
         super(IosBaseCompiler, self).ospf(node)
+        for interface in node.interfaces:
+            # get ospf info for this id
+            ospf_link = self.anm['ospf'].edge(interface._edge_id) # find link in OSPF with this ID
+            if ospf_link:
+                area = str(ospf_link.area)
+                interface['ospf'] = {
+                        'area': area,
+                        'process_id': node.ospf.process_id, # from super ospf config
+                        }
         
     def vrf(self, node):
         vrf_node = self.anm['vrf'].node(node)
@@ -392,16 +417,6 @@ class NxOsCompiler(IosBaseCompiler):
 
     def ospf(self, node):
         super(NxOsCompiler, self).ospf(node)
-        for interface in node.interfaces:
-            # get ospf info for this id
-            ospf_link = self.anm['ospf'].edge(interface._edge_id) # find link in OSPF with this ID
-            if ospf_link:
-                area = str(ospf_link.area)
-                interface['ospf'] = {
-                        'area': area,
-                        'process_id': node.ospf.process_id, # from super ospf config
-                        }
-
         #TODO: configure OSPF on loopback like example
 
 
